/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef IDLC_C_GEN_C_GEN_BASE_H_
#define IDLC_C_GEN_C_GEN_BASE_H_

#include <memory>
#include <string>
#include <map>

#include "idlc/type.h"
#include "idlc/structure.h"
#include "idlc/generator.h"

namespace tidl {

class CGeneratorBase : public Generator {
 public:
  explicit CGeneratorBase(std::shared_ptr<Document> doc);
  virtual ~CGeneratorBase() = default;

  std::string Trim(const std::string& str);
  std::string SmartIndent(std::string lines);
  std::string Tab(int cnt);
  std::string NLine(int cnt);
  std::string ConvertTypeToString(ParameterType::Direction direction,
                                  const BaseType& type, bool bconst = true);
  std::string GetFullNameFromType(const BaseType& type);
  std::string GetParcelParamTypeString(const BaseType& type,
                                       bool is_pointer = true);
  std::string GetReturnTypeString(const BaseType& type);
  std::string GetParamTypeString(ParameterType::Direction direction,
                                 const BaseType& type);
  std::string GetStringFromElementType(const BaseType& type);
  std::string GetErrorValue(const BaseType& type);
  void AddStructureFromType(const BaseType& type);
  const std::map<std::string, std::unique_ptr<Structure>>& GetStructures(void) {
    return structures_;
  }

  void GenVersion(std::ofstream& stream);
  void GenIncludeDefaultHeaders(std::ofstream& stream, bool body = true);
  void GenGNUSourceDefinition(std::ofstream& stream);
  std::string GetStructIdWithNamespace(const Structure& st);
  std::string GetInterfaceIdWithNamespace(const Interface& inf);
  std::string GetHandlePrefix();

 private:
  bool StructureExist(const Structure& st);

 private:
  std::map<std::string, std::unique_ptr<Structure>> structures_;
  std::map<std::string, std::string> type_map_;
};

}  // namespace tidl

#endif  // IDLC_C_GEN_C_GEN_BASE_H_
