/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef IDLC_CS_GEN_CS_GEN_BASE_CB_H_
#define IDLC_CS_GEN_CS_GEN_BASE_CB_H_

const char CB_CALLBACK_BASE[] =
R"__cs_cb(
            public abstract class CallbackBase
            {
                internal int Id;
                internal int SeqId;
                internal bool Once;
                private static volatile int _seqNum = 0;

                public string Tag
                {
                    get
                    {
                        return Id.ToString() + "::" + SeqId.ToString();
                    }
                }

                public CallbackBase(int delegateId, bool once)
                {
                    Id = delegateId;
                    SeqId = _seqNum++;
                    Once = once;
                }

                internal virtual void OnReceivedEvent(Parcel p) {}

                internal static void Serialize(Parcel h, CallbackBase param)
                {
                    h.WriteInt(param.Id);
                    h.WriteInt(param.SeqId);
                    h.WriteBool(param.Once);
                }

                internal static void Deserialize(Parcel h, CallbackBase param)
                {
                    param.Id = h.ReadInt();
                    param.SeqId = h.ReadInt();
                    param.Once = h.ReadBool();
                }
            }

)__cs_cb";

const char CB_CALLBACK_CTOR_PROXY[] =
R"__cs_cb(                public $$(bool once = false) : base((int)DelegateId.$$, once)
                {
                }
)__cs_cb";

const char CB_CALLBACK_CTOR_STUB[] =
R"__cs_cb(                internal $$(Port port, WeakReference service) : base((int)DelegateId.$$, false)
                {
                    _port = port;
                    _service = service;
                }
)__cs_cb";

const char CB_CALLBACK_STUB_MEMBERS[] =
R"__cs_cb(                private Port _port;
                private WeakReference _service;
                private bool _valid = true;

)__cs_cb";

const char CB_CALLBACK_INVOKE_METHOD[] =
R"__cs_cb(
                public void Invoke($$)
                {
                    if (_port == null)
                        throw new NotConnectedSocketException();
                    if (!_service.IsAlive)
                        throw new InvalidProtocolException();
                    if (Once && !_valid)
                        throw new InvalidCallbackException();

                    using (var p = new Parcel())
                    {
                        p.WriteInt((int)MethodId.__Callback);
                        Serialize(p, this);
$$
                        // Send
                        p.Send(_port);
                        _valid = false;
                    }
                }
)__cs_cb";

const char CB_PROPERTY[] = R"__cs_cb(        public $$ $$ { get; set; }
)__cs_cb";

const char CB_CTOR[] = R"__cs_cb(
        public $$()
        {
$$
        }
)__cs_cb";

const char CB_WRITE_BUNDLE[] =
R"__cs_cb(
                if (param.$$ != null)
                {
                    h.WriteBundle(param.$$);
                }
                else
                {
                    h.WriteBundle(new Bundle());
                }
)__cs_cb";

#endif  // IDLC_CS_GEN_CS_GEN_BASE_CB_H_
