/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef IDLC_DECLARATION_H_
#define IDLC_DECLARATION_H_

#include <string>
#include <list>
#include <memory>

#include "idlc/type.h"
#include "idlc/parameter.h"

namespace tidl {

class Declaration {
 public:
  enum class MethodType {
    SYNC,
    ASYNC,
    DELEGATE
  };

  Declaration(std::string id, BaseType* ret_type, Parameters* params,
              std::string comments, unsigned line,
              MethodType mtype = MethodType::SYNC);

  const std::string& GetID() const;
  const BaseType& GetType() const;
  const Parameters& GetParameters() const;
  MethodType GetMethodType() const { return mtype_; }
  const unsigned GetLine() const;
  const std::string& GetComments() const;

 private:
  std::string id_;
  std::unique_ptr<BaseType> ret_type_;
  std::unique_ptr<Parameters> params_;
  std::string comments_;
  unsigned line_;
  MethodType mtype_;
};

class Declarations {
 public:
  void Add(Declaration* decl);
  const std::list<std::unique_ptr<Declaration>>& GetDecls() const;
  bool Exist(Declaration* decl) const;

 private:
  std::list<std::unique_ptr<Declaration>> decls_;
};

}  // namespace tidl

#endif  // IDLC_DECLARATION_H_
