/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <utility>

#include "idlc/parameter.h"
#include "idlc/type.h"

namespace tidl {

Parameter::Parameter(std::string id, ParameterType* type, unsigned line)
    : type_(type), id_(std::move(id)), line_(line) {}

const std::string& Parameter::GetID() const {
  return id_;
}

const ParameterType& Parameter::GetParameterType() const {
  return *type_;
}

const unsigned Parameter::GetLine() const {
  return line_;
}

void Parameters::Add(Parameter* param) {
  params_.emplace_back(param);
}

const std::list<std::unique_ptr<Parameter>>& Parameters::GetParams() const {
  return params_;
}

bool Parameters::Exist(Parameter* param) const {
  for (auto& p : params_) {
    if (p->GetID() == param->GetID())
      return true;
  }

  return false;
}

}  // namespace tidl

